%---------------------------------------------------
% f07_ar
% Procesy AR -- identifikace
%---------------------------------------------------
close all
clear all
clc

% spise kratsi rada ale nizsi rozptyl bileho sumu

y=[-0.0414
    0.0118
   -0.0250
   -0.0260
   -0.0802
   -0.0672
   -0.0648
   -0.1513
   -0.0630
   -0.0307
   -0.0916
   -0.0694
   -0.1464
   -0.1567
   -0.1382
   -0.1211
   -0.1242
   -0.0204
   -0.0202
    0.0626
    0.1144
    0.0996
    0.1219
    0.0771
    0.0075
   -0.0014
    0.0275
    0.0368
   -0.0112
   -0.0105
   -0.0056
   -0.0562
   -0.0743
   -0.1435
   -0.0875
   -0.1739
   -0.1157
   -0.1377
   -0.1285
   -0.0706
   -0.1260
   -0.0683
   -0.0826
   -0.1313
   -0.1454
   -0.1328
   -0.0524
   -0.0542
    0.0087
   -0.0979
   -0.0070
   -0.0968
   -0.0636
    0.0063
   -0.0494
   -0.0793
    0.0722
    0.0094
    0.0254
    0.0520
    0.0352
   -0.0212
   -0.0559
   -0.0385
   -0.0543
   -0.0689
    0.0013
    0.0020
    0.0181
    0.0371
   -0.0385
    0.0206
   -0.0391
   -0.0688
    0.0523
    0.0855
    0.0242
   -0.0552
   -0.0370
    0.0344
    0.0896
    0.0471
    0.0917
    0.0890
    0.0623
    0.0345
   -0.0294
   -0.0632
   -0.1043
   -0.0130
    0.0221
    0.1030
    0.1265
    0.1044
    0.0440
    0.0082
   -0.0003
    0.0157
   -0.0109
    0.0510];
    
%---------------------------------------------------
% Obrazek casove rady
%---------------------------------------------------
figure
plot(y)
hold on
plot(mean(y)*ones(size(y)),'g')
plot(zeros(size(y)),'k:')
title('Pozorovana rada')
legend('data','prumer','zero line')
% zjistujeme zda ma proces nenulovou stredni hodnotu, v tom pripade bychom
% proces modelovali jako ARX 

%---------------------------------------------------
% Autokor. a parc. autokor. fce
%---------------------------------------------------
figure
subplot(2,1,1)
plotacf(y,29)
title('Autokorelacni funkce')
subplot(2,1,2)
plotpacf(y,29)
title('Parcialni autokor. fce')

% Autokorelacni funkce casove rady klesa exponencialne do Bartletova aproximacniho 
% intervalu, ve kterem ji muzeme povazovat za nulovou. Nechova se zde vsak nahodne, ale
% ma zde  nenahodny prubeh. 
% U parcialni autokorelacni funkce je videt, ze statisticky
% vyznamne je pouze jedno zpozdeni. Klesa rychle do Quenouilleova intervalu.
% Bude se nejspis jednat o proces AR(1).  
% U funkce PACF se pri zpozdeni 5 vyskakuje PACF z Quenouilleova intervalu. To 
% muze byt zpusobeno sumem obsazenym v rade.

%---------------------------------------------------
% Odhad
%---------------------------------------------------
% Odhad parametru modelu pomoci funkce ARMAX -- viz help a documentation
% AR(1) = ARMA(1,0), tedy funkci ARMAX zadame krome dat "y" jeste matici s
% rady procesu [1 0]
th=armax(y,[1 0]); % AR1

%---------------------------------------------------
% Zobrazeni vysledku
%---------------------------------------------------
% funkce PRESENT zobrazi vysledky odhadu parametru modelu, smerodatne
% odchylky parametru modelu, ztratovou funkci a Akaikeho kriterium FPE
% (final prediction error).
% Vzdy budeme volit model s parametry statisticky vyznamnymi - to bude
% jakasi pripustna mnozina modelu, vybereme ten s minimalni chybou (FPE).
%
% Vyznamnost parametru otestujeme pomoci t-testu. Podil absolutni hodnoty
% parametru a jeho smerodatne odchylky ma byt vetsi nez prislusny Studentuv
% kvantil, coz je pro velka n zhruba 2. 
% Staci tedy zjistit, je-li parametr vic nez 2x tak velky jako jeho std.
% Nebo, pokud neumime nasobit desetinna cisla dvema, na to pouzijeme
% nejakou funkci, ktera to bude pocitat za nas :o)
present(th)

%---------------------------------------------------
% Jine modely
%---------------------------------------------------
% pro jistotu si odhadneme jine, blizke procesy abychom nasli nejnizsi FPE
th2=armax(y,[2 0]); % AR2
present(th2) % druhy parametr je statisticky nevyznamny (+-std), absolutni hodnota parametru musi byt priblizne 2x sm. odchylka
th3=armax(y,[1 1]); % ARMA11
present(th3) % parametr MA casti je statisticky nevyznamny
th4=armax(y,[0 1]); % MA1
present(th4) % FPE je vyssi nez u AR1
% => ukazuje se ze nejlepsi je opravdu AR1 (pokud by ovsem nevysla
% verifikace, tak bychom museli zvolit jiny proces (s vyssim FPE)

pars = th.ParameterVector; % vytazeni vektoru parametru z "th"

%---------------------------------------------------
% Verifikace - predvyrobene funkce (nutno zkopirovat k sobe)
%---------------------------------------------------
% a) verifikace parametru
p_sig = par_sig(th) % help par_sig pro vice info
% b) verifikace rezidui
[sig,tests,sk]=res_sig(th,y) %help res_sig pro vice info

%---------------------------------------------------
% Stabilita
%---------------------------------------------------
% Stabilitu otestujeme pomoci funkce PZMAP, ktera pocita/kresli nuly a poly
% systemu, chceme je vsechny uvnitr jednotkove kruznice
figure
pzmap(th)

%---------------------------------------------------
% Predikce
%---------------------------------------------------
% na vypocet predikci slouzi funkce PREDICT, pro predikci modelu v theta
% formatu a pro dana data na K kroku dopredu pouzivame prikaz:
% YP = PREDICT(MODEL,DATA,K)

% K je predikcni horizont, k predikci hodnoty na "K" obdobi dopredu se
% pouziji veskera data dostupna do casu "t-K".

% Vidime, jak se predikce na vice kroku dopredu meni - cim dal tim min se
% podobaji skutecnym hodnotam. Na rozdil od MA procesu ale predikce
% konverguji k nule postupne -- modely maji "dlouhou" pamet.

% Jednokrokove predikce modelu
figure
yp1=predict(th,y,1);
plot(y,'b')
hold on
plot(yp1,'r');
title('Predpoved na 1 krok dopredu')
hold off

% Dvoukrokove predikce modelu
figure
yp2=predict(th,y,2);
plot(y,'b')
hold on
plot(yp2,'r');
title('Predpoved na 2 kroky dopredu')
hold off

% Trikrokove predikce modelu
figure
yp3=predict(th,y,3);
plot(y,'b')
hold on
plot(yp3,'r');
title('Predpoved na 3 kroky dopredu')
hold off

%---------------------------------------------------
% Predpoved do budoucna
%---------------------------------------------------
pars = - pars; % je tam jen jeden parametr a to je AR casti (prevadime na druhou stranu rovnice)
h_p  = 15 ; % horizont predikce

for i=1:h_p
    y_p(i)=pars.^i*y(end);
end

% kresleni predpovedi
figure
plot(y)
hold on
plot(length(y)+1:length(y)+1+length(y_p)-1,y_p,'r.')

%---------------------------------------------------
% konfidencni intervaly predpovedi
%---------------------------------------------------
e=pe(y,th); % rezidua
s2e = var(e); % odhad rozptylu nhodn sloky
s2et = 0; % odhad rozptylu chyby predpovedi
for i=1:h_p
    s2et(i) = 0;
    for j=1:i
    s2et(i) = s2et(i) + pars^(2*j-2);
    end
end
s2et = s2et * s2e;
tt = tinv(1-.05/2,length(y));
y_ph = y_p + tt*sqrt(s2et);
y_pd = y_p - tt*sqrt(s2et);

plot(length(y)+1:length(y)+1+length(y_p)-1,y_ph,'g')
plot(length(y)+1:length(y)+1+length(y_p)-1,y_pd,'g')
title('Predpoved do budoucna a konfidencni intervaly predpovedi')
