% Proces ARIMA(3,1,2) -- simulace dat/estimace
%========================================================================
%         BOX-JENKINSOVY ARMA MODELY PRO STACIONARNI CASOVE RADY
% Systematicky postup identifikace, analyzy chovani a stability
%              (vyzaduje SYSTEM IDENTIFICATION TOOLBOX a CONTROL TOOLBOX)
%========================================================================
                                                                         
close all
clear all

format
disp('   Priklad ARMA - modelu'); 
disp('   ~~~~~~~~~~~~~~~~~~~~~');
disp('   ');
disp('   Simulace modelu');
disp('   ');
disp('   y(t) - 1.5*y(t-1) + 0.9*y(t-2) - 0.1*y(t-3) = 0.045*u(t) + e(t) - 0.5*e(t-1) + 0.25*e(t-2)');
disp('  '); 

% Pocet pozorovani
  N=412;
% kalendarni cas v mesicich
  tsim = (1960+(1/12):1/12:1960+(N/12))';
% A R M A(2,2) - model
%*********************
% Vektory parametru
  A = [ 1 -1.5 0.9 -0.1];
  B = 0.045;
  C = [ 1 -0.5 0.25];

% Sestaveni theta formatu 
  th0  = poly2th(A, B, C);

% Simulace "pozorovane" casove rady
% generator - rezidua
  esim = randn(N-1,1)*2.0; %<===<<<

% "pozorovana" casova rada
% ************************ 
  u  = ones(N-1,1);
  ue = [u,esim];
  w1_sim = idsim(ue,th0);
  y(1) = 22.5;
  y = [y(1); y(1) + cumsum(w1_sim)] ;
  
% plot(tsim,y,tsim,w1_sim)
  
  w1 = [0;diff(y)];

% plot(tsim,w1_sim,'r+-',tsim,w1,'bx')

%>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

% Autokor. a parc. autokor. fce
 figure
 set(gcf,'Name','Autokorelacncni a parcialni autokorelacni funkce (ACF a PACF)') 
 subplot(2,1,1)
 plotacf(w1,40)
 
 title('Autokorelacni funkce pro w^1')
 subplot(2,1,2)
 plotpacf(w1,40)
 title('Parcialni autokorelacni funkce pro w^1')
  
  N  = length(w1);
  t  = (1960+(1/12):1/12:1960+(N/12))';
% data  
  u  = ones(N-1,1);
  z  = [w1(2:end), u]; 
 % odhad ARMAX modelu pro [w1 u] 
  na = 3;
  nb = 1;
  nc = 2;
  nk = 0;
  th = armax(z, [na, nb, nc, nk]);
disp('   Odhad parametru ARIMA modelu')
disp('========================================================================')  
  present(th)
disp('========================================================================')   
% vypocet a testy residuii ARMAX modelu
  figure
  set(gcf,'Name','Korelacni funkce rezidui ARIMA(3,1,2)-modelu');
  e = resid(z, th);
  hold off
 
% simulace casove rady
  ue = [u e];
  
  w1_arma = idsim(ue,th);
  
% odstraneni dynamicke ulice 
  w1_arma = w1_arma-mean(w1_arma-w1(2:end));

  y_arima  = [y(1);y(1)+cumsum(w1_arma)];
  
  w1_arma = [0;w1_arma];
  
figure
set(gcf,'Name','Rady w1 a y simulovane ARIMA(3,1,2)-modelem');
subplot(211)
plot(t,w1,'b.-',t,w1_arma,'rx-')
legend('data','ARIMA(3,1,2)')
title('w^1_t = {\Delta}y_t');
subplot(212)
plot(t,y,'b-',t,y_arima,'r-')
title('y_t ... simulace ARIMA(3,1,2)');
legend('data','ARIMA(3,1,2)')

% predikce 1 krok vpred
  w1_arma1 = predict(th,z,1);
  
  w1_arma1 = w1_arma1-mean(w1_arma1-w1(2:end));

  y_arima1  = [y(1);y(1)+cumsum(w1_arma1)];
  y_arima1 = y_arima1-mean(y_arima1-y);

  w1_arma1 = [0;w1_arma1];
  
figure
set(gcf,'Name','Rady w1 a y predikovane ARIMA(3,1,2)-modelem');
subplot(211)
plot(t,w1,'b.-',t,w1_arma1,'rx-')
legend('data','ARIMA(3,1,2)')
title('w^1_t = {\Delta}y_t ... predikce na 1 obdobi');
subplot(212)
plot(t,y,'b-',t,y_arima1,'r-')
title('y_t ... predikce na 1 obdobi');
legend('data','ARIMA(3,1,2)')

% predikce 2 kroky vpred
  w1_arma2  = predict(th,z, 2);
  y_arima2  = [y(1);y(1)+ cumsum(w1_arma2)];
  
% predikce 3 kroky vpred  
  w1_arma3 = predict(th,z, 6);
  y_arima3  = [y(1);y(1)+cumsum(w1_arma3)];

figure
set(gcf,'Name','Rady w1 a y predikovane na 2 a 3 obdobi');
subplot(211)
plot(t,y,'b-',t,y_arima2,'r-')
title('y_t ... predikce na 2 obdobi');
legend('data','ARIMA(3,1,2)')  
subplot(212)
plot(t,y,'b-',t,y_arima3,'r-')
title('y_t ... predikce na 3 obdobi');
legend('data','ARIMA(3,1,2)')

e    = [0; e]; 
esim = [0; esim];
figure
set(gcf,'Name','Sok e');
subplot(211)
plot(t(1:end),esim,'b.-')
hold on
plot(t(1:end),e,'rx-')
hold on
plot(t(1:end),zeros(size(esim)),'r:')
hold on
plot(t(1:end),[ones(size(esim))*2*std(esim),-ones(size(esim))*2*std(esim)],'r:')
title('Sok e_t ~ N(\mu,\sigma^2), \mu = 0, \sigma^2 = 1 ... W.N. (bily sum)');
legend('e\_teoretic_t','e\_model_t')

subplot(223)
hist(esim)
title('Histogram e\_teoretic');
subplot(224)
hist(e)
title('Histogram e\_model');

% teoreticka odezva w1
  [w1_imp0,timp0] = impulse(th0);
  
% impulsni odezva realizace casove rady w1
  [w1_imp,timp]  = impulse(th);

% teoreticka odezva y
  [y_stp0,timp0] = step(th0);  
% odezva y
  [y_stp,timp]  = step(th);  

figure
set(gcf,'Name','Oodezvy ARIMA(3,1,2)-modelu ma jednotkovy nahodny sok'); 
subplot(2,1,1)
plot(timp0(1:end)/12,w1_imp0(1:end),'r*-')
hold on
plot(timp(1:end)/12,w1_imp(1:end),'b.-')
hold on
stem(timp(1:end)/12,(w1_imp(1:end)),'.:')
title(['Odezva {\Delta}y_t na jednotkovy nahodny sok, ARIMA(3,1,2)-model, stredni hodnota {\Delta}y_t je y\_mean = ',num2str(mean(diff(y))),]); 
xlabel('k - zpozdeni (rok)');
legend('teoreticka odezva', 'odezva dane realizace rady')
ylabel('{\Delta}y_t')

subplot(2,1,2)
plot(timp0(1:end)/12,y_stp0,'r*-'); % cumsum(w1_imp0(1:end)),'r*-')
hold on
plot(timp(1:end)/12,y_stp,'b.-');  % cumsum(w1_imp(1:end)),'b.-')
hold on
stem(timp(1:end)/12,y_stp,'b.:');  % cumsum(w1_imp(1:end)),'.:')
title('Odezva y_t na jednotkovy nahodny sok, ARIMA(3,1,2)-model'); 
xlabel('k - zpozdeni (rok)');
ylabel('y_t')
legend('teoreticka odezva', 'odezva dane realizace rady')

K=20;

% Autokorelacni funkce rezidui modelu
  r_e = covf(e,K)/max(covf(e,K));
% Boxuv-Priceuv test
  T = N-1;
  Q = T*sum(r_e(2:end).^2);
% Kriticka hodnota ci^2  
  chi2 = chi2inv(0.05,K-3-2);
% Standardni odchylka disturbance
  sig_e = sqrt(sum(e.^2)/(N-3-1-2));
  %........................................
disp('   ');
disp('   Boxuv a Priceuv test');
disp('   ~~~~~~~~~~~~~~~~~~~~');
disp(['   Q       = ',num2str(Q),])
disp('   Kriticka hodnota chi^2 testu');
disp(['   chi^2   = ',num2str(chi2),])
disp('   ');
disp('   Standardni odchylka disturbance');
disp(['   sigma_e = ',num2str(sig_e),])
disp('   ');
%........................................
% Parametry odhadnute ARMAX modelem
  theta = th2par(th);
  
  T  = length(w1);
  a  = -theta(1:3); 
  b0 =  theta(4);
  c  =  theta(5:6);
% Predpoved w1, Lpr = 6
  T  = length(w1);
  Lpr = 6;
  
  w1_T(1) = a(1)*w1(T)   + a(2)*w1(T-1) + a(3)*w1(T-2) + b0 + c(1)*e(T) + c(2)*e(T-1);
  w1_T(2) = a(1)*w1_T(1) + a(2)*w1(T)   + a(3)*w1(T-1) + b0 + c(2)*e(T);
  w1_T(3) = a(1)*w1_T(2) + a(2)*w1_T(1) + a(3)*w1(T)   + b0;
  w1_T(4) = a(1)*w1_T(3) + a(2)*w1_T(2) + a(3)*w1_T(1) + b0;
  w1_T(5) = a(1)*w1_T(4) + a(2)*w1_T(3) + a(3)*w1_T(2) + b0;
  w1_T(6) = a(1)*w1_T(5) + a(2)*w1_T(4) + a(3)*w1_T(3) + b0;
  
% Intervaly spolehlivosti pro predpoved w1 
  w1_T_int  = [ w1_T' + sig_e,    w1_T' - sig_e ];
  w1_T_2int = [ w1_T' + 2*sig_e,  w1_T' - 2*sig_e ];
 % Predpoved a intervaly spolehlivosti pro y 
  y_T       = [y(end) + cumsum(w1_T(:)')];
  y_T_int   = [y(end) + cumsum(w1_T' + sig_e),    y(end) + cumsum(w1_T' - sig_e)];
  y_T_2int  = [y(end) + cumsum(w1_T' + 2*sig_e),  y(end) + cumsum(w1_T' - 2*sig_e)];
  
tpr = (t(end)+1/12:1/12:t(end)+Lpr/12);
figure
set(gcf,'Name','Predpovedi a intervaly splehlivosti')
subplot(2,1,1)
plot(t(301:end),w1(301:end),'b.-')
hold on
plot(tpr,w1_T,'r*-')
hold on
plot(tpr,w1_T_int,'k:')
hold on
plot(tpr,w1_T_2int,'k-')
title(['w1_T(l_p) ... predpoved ARIMA(3,1,2)-modelu (l_p = 1, 2, ..., L_{pr}; L_{pr} = ',num2str(Lpr),')']) 

subplot(2,1,2)
plot(t(301:end),y(301:end),'b.-')
hold on
plot(tpr,y_T,'r*-')
hold on
plot(tpr,y_T_int,'k:')
hold on
plot(tpr,y_T_2int,'k-')
title(['y_T(l_p) ... predpoved ARIMA(3,1,2)-modelu (l_p = 1, 2, ..., L_{pr}; L_{pr} = ',num2str(Lpr),')']) 
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Control Toolbox
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Stavovy model
% ~~~~~~~~~~~~~
  ARIMA312ss = ss(th);

figure
set(gcf,'Name','Prenosove "nuly" a "poly" ARIMA(3,1,2)-modelu')

% Stabilita procesu - jednotkova kruznice
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  axis square
  pzmap(ARIMA312ss);
  zgrid;
  axis([-1, 1, -1, 1])
  
% Vlastni cisla
disp('  Vlastni cisla matice ARIMA(3,1,2)-modelu')
eigval = eig(ARIMA312ss);
disp(eigval)

% Nuly a poly
% ~~~~~~~~~~~
  [Z, P]=pzmap(ARIMA312ss);
  
  disp('  Prenosove "poly" a "nuly" ARIMA(3,2,1)-modelu')
  disp('   "nuly"');
  disp(Z)
  disp('   "poly"');
  disp(P)
  disp('   ');


